from typing import Dict

from base_request import RjBaseRequest

import logging

logger = logging.getLogger(__name__)


class UserAuth(RjBaseRequest):

    def signup(self,
               name: str,
               last_name: str,
               email: str,
               password: str
               ) -> bool:
        """
        Signup a new usr on radio javan.
        Mobile signup requires no special validation.

        :param name: name of the new user
        :param last_name: last name of the user
        :param email: email address
        :param password: a random password!
        :return: True if account created
        :rtype: bool
        """

        address = "https://d1gyyseacf3czm.cloudfront.net/api2/signup_mobile"
        post_data = {
                     "firstname": name,
                     "lastname": last_name,
                     "email": email,
                     "email_confirm": email,
                     "password": password,
                     "username": email.split("@")[0]+"."+last_name
                     }
        self.set_ajax()

        # init session
        result = self.post(address, post_data, True, enable_proxy=True, cookies={"_rj_web": "BAh7CUkiD3Nlc3Npb25faWQGOgZFVEkiJWMzN2RhOTg1ZTU4MmQyMzhjZDgzMmY1ZDVlZDQ0YTQ1BjsAVEkiDGdlb2luZm8GOwBGewg6D2lwX2FkZHJlc3NJIhI1LjExNy4xNTcuMjI1BjsAVDoMZXhwaXJlc0l1OglUaW1lDcmBHoCMJ78GCjoLb2Zmc2V0af6wuToJem9uZUkiCEVTVAY7AEY6DW5hbm9fbnVtaQJpAzoNbmFub19kZW5pBjoNc3VibWljcm8iB4cwOgdkYnsJOhFjb3VudHJ5X2NvZGVJIgdJUgY7AFQ6EWNvdW50cnlfbmFtZUkiCUlyYW4GOwBUOg1sYXRpdHVkZWYLMzUuNjk4Og5sb25naXR1ZGVmDDUxLjQxMTVJIg1hcHBfaG9tZQY7AEZUSSISYWRicmVha19jb3VudAY7AEZpBw==--bec9fb194d273879cbb6b8d88405b010054f3121;"})
        return result

    def login(self,
              username: str,
              password: str
              ) -> Dict:
        """
        Login to api service

        :param username: username
        :param password: password of user
        :return: data returned by server. Usually contains success parameter and a message
        :rtype: Dict
        """

        address = "https://r-j-app-desk.com/api2/login"
        data = {"login_email": username,
                "login_password": password
                }
        return self.post(address, data, True, False)

    def user_profile(self) -> Dict:
        """
        Dictionary contains data for the user profile and suggestions

        :return: User dashboard data
        :rtype: Dict
        """

        address = "https://r-j-app-desk.com/api2/user_profile?stats=1"
        res = self.get(address, True)
        return res

    def get_subscription(self) -> Dict:
        """
        Get user subscription information

        :return: Dict of user subscription
        :rtype: Dict
        """

        return self.get("https://r-j-app-desk.com/api2/user_subscription", True)



from typing import Dict

from base_request import RjBaseRequest


class MP3(RjBaseRequest):
    """
    Class to operate on mp3 files
    """

    def get_playlist(self) -> Dict:
        """
        Get user active play lost

        :return:
        :rtype: Dict
        """

        return self.get("https://r-j-app-desk.com/api2/playlists_dash", True)

    def info(self, mp3_id: int):
        """

        :param mp3_id:
        :return:
        """

        return self.get(f"https://r-j-app-desk.com/api2/mp3?id={mp3_id}", True)

    def vote(self, mp3_id: int):
        res = self.get(f"https://r-j-app-desk.com/api2/mp3_vote?id={mp3_id}&type=mp3&vote=5", True)
        return res



import logging

import requests
from bs4 import BeautifulSoup

logger = logging.getLogger(__name__)


class RjBaseRequest(object):

    # We need to change the port for all sub-classes.
    proxy_port = 2000

    def __init__(self, session=None):
        if session is None:
            self.session = requests.Session()
            self.session.headers["User-Agent"] = "Radio Javan/16.9.1 (iPhone; iOS 15.2; Scale/3.00) com.RadioJavan.RJ"
            self.session.headers["Sec-Fetch-Site"] = "cross-site"
            self.session.headers["Sec-Fetch-Mode"] = "cors"
            self.session.headers["Sec-Fetch-Dest"] = "empty"
            self.session.headers["Sec-Fetch-Dest"] = "empty"
            self.session.headers["Referer"] = "https://d2ztbi9tgqidw4.cloudfront.net/"
            self.session.headers["Accept-Language"] = "en-US"
            # self.session.get("https://r-j-app-desk.com/api2/app_config")
        else:
            self.session = session

    def _get_next_proxy_port(self):
        if self.proxy_port >= 2179:
            self.proxy_port = 2000
        return self.proxy_port

    def read_config(self):
        return self.session.get("https://r-j-app-desk.com/api2/app_config").json()

    def get_session(self):
        return self.session

    def download_file(self, address):
        local_filename = address.split('/')[-1]

        # NOTE the stream=True parameter below
        with self.session.get(address, stream=True) as r:
            r.raise_for_status()
            with open(local_filename, 'wb') as f:
                for chunk in r.iter_content(chunk_size=8192):
                    f.write(chunk)
        return local_filename

    def set_ajax(self):
        self.session.headers["X-Requested-With"] = "XMLHttpRequest"

    def unset_ajax(self):
        del self.session.headers["X-Requested-With"]

    def extract_token(self, address):
        data = self.get(address, False, True)
        if not data:
            logger.error("Load error! No data returned")
            return False, None
        html_data = BeautifulSoup(data, 'html.parser')
        token = html_data.find_all("input", {'name': 'authenticity_token'})
        if len(token) < 1:
            logger.error("Load Error! Token not found.")
            return False, None
        token_data = token[0].get("value")
        if not token_data:
            logger.error("Load Error! Token data is not valid.")
            return False, None
        return True, token_data

    def _get_proxy(self):
        px = self._get_next_proxy_port()
        res = {"http": f"socks4://127.0.0.1:{px}", "https": f"socks4://127.0.0.1:{px}"}
        return res

    def get(self, address, as_json=False, enable_proxy=False):
        if enable_proxy:
            response = self.session.get(address, proxies=self._get_proxy())
        else:
            response = self.session.get(address)
        if response.status_code == 200:
            if as_json:
                return response.json()
            return response.content
        logger.warning(f"Expected status 200 but got other data : {response.status_code}")

    def post(self, address, data: dict, as_json=False, as_ajax=False, enable_proxy=False, cookies={}):
        if as_ajax:
            self.set_ajax()
        try:
            if enable_proxy:
                response = self.session.post(address, data=data, proxies=self._get_proxy(), cookies=cookies)
            else:
                response = self.session.post(address, data=data, cookies=cookies)
        except:
            return {}
        if as_ajax:
            self.unset_ajax()
        if response.status_code == 200:
            if as_json:
                try:
                    return response.json()
                except Exception as e:
                    logger.error(e.args)
                    return {}
            return response.content
        logger.warning(f"Expected status 200 but got other data : {response.status_code}")
